#ifndef _GAME_H_
#define _GAME_H_

#include <qcanvas.h>
#include "mapa.h"
#include "sprite.h"

class Scene {
	public:
	virtual		~Scene() {}
	virtual	bool	enter() { return true; }
	virtual	void	leave() {}
	virtual	void	advance() {}
	};

class Game: public QCanvas { Q_OBJECT
	public:
			Game		();

		void	deleteAllItems	();

		void	setLimits	(int x0, int y0, int x1, int y1);

		void	setTilesGr	(QPixmap* tiles, int tile_dx, int tile_dy);
		CTile*	ptile		(int x, int y);
		bool	setMapTile	(int x, int y, CTile ct);
		void	setMapa		(Mapa *m);
		bool	mapCollides	(int x, int y);
		int	mapCollides	(const QRect &rec);

		void	setScene	(Scene*);
		Scene*	scene		() const { return cur_scene; }

		unsigned	cur_frame;
		int		ntiles;

	public slots:
		void		advance	();

	protected:
		void		deleteKilled();

		QList<Sprite>	delitems;
		Scene*		cur_scene;
		Scene*		next_scene;
		Mapa*		mapa;
		int		limit_x0, limit_y0,
				limit_x1, limit_y1;
		QPixmap*	tiles;
		int		tile_dx, tile_dy;

		friend class	Sprite;
	};

#define	CM_CENTER_UP	0x01
#define	CM_CENTER_DOWN	0x02
#define	CM_CENTER_LEFT	0x04
#define	CM_CENTER_RIGHT	0x08
#define	CM_LEFT_UP	0x10	
#define	CM_RIGHT_UP	0x20
#define	CM_LEFT_DOWN	0x40
#define	CM_RIGHT_DOWN	0x80

#define	CM_UP		(CM_CENTER_UP|CM_LEFT_UP|CM_RIGHT_UP)
#define	CM_DOWN		(CM_CENTER_DOWN|CM_LEFT_DOWN|CM_RIGHT_DOWN)
#define	CM_LEFT		(CM_CENTER_LEFT|CM_LEFT_UP|CM_LEFT_DOWN)
#define	CM_RIGHT	(CM_CENTER_RIGHT|CM_RIGHT_UP|CM_RIGHT_DOWN)

#endif
