#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <qtimer.h>
#include "game.h"

Game::Game ()
{
cur_frame = 0;
cur_scene = next_scene = 0;
setBackgroundColor(QColor(0,0,0));
tiles = 0;
#if QT_VERSION < 300
setAdvancePeriod(10);
setUpdatePeriod(10);
#else
setAdvancePeriod(-1);
setUpdatePeriod(-1);
QTimer *timer = new QTimer(this);
connect( timer, SIGNAL(timeout()), this, SLOT(advance()) );
timer->start(10);
#endif
}

void Game::setScene (Scene* sce)
{ next_scene = sce; }

void Game::advance ()
{
cur_frame++;
if(cur_scene != next_scene)
	{
	if(cur_scene)
		cur_scene->leave();
	if(next_scene)
		if(next_scene->enter())
			cur_scene = next_scene;
		else	next_scene = 0;
	update();
	}
if(cur_scene)
	cur_scene->advance();
deleteKilled();
QCanvas::advance();
}

void Game::deleteKilled()
{
while(!delitems.isEmpty())
	delete delitems.take(0);
}

void Game::deleteAllItems()
{
deleteKilled();
QCanvasItemList list = allItems ();
QCanvasItemList::Iterator it;
for( it = list.begin(); it != list.end(); ++it )
	{
	QCanvasItem *item = *it;
	delete item;
	}
}

void Game::setTilesGr (QPixmap* qpt, int tdx, int tdy)
{
tiles = qpt;
ntiles = (qpt->width()/tdx)*(qpt->height()/tdy);
tile_dx = tdx;
tile_dy = tdy;
}

void Game::setMapa (Mapa *m)
{
mapa = m;
if(m && tiles)
	{
	int i, j;

	limit_x0 = limit_y0 = 0;
	limit_x1 = mapa->width*tile_dx;
	limit_y1 = mapa->height*tile_dy;
	resize(limit_x1, limit_y1);

	setTiles(*tiles, mapa->width, mapa->height, tile_dx, tile_dx);
	for(i=0; i<mapa->width; i++)
		for(j=0; j<mapa->height; j++)
			setTile(i,j, *mapa->tile(i,j) & mapa->mask_tile);
	}
}

void Game::setLimits (int x0, int y0, int x1, int y1)
{
limit_x0 = x0;
limit_x1 = x1;
limit_y0 = y0;
limit_y1 = y1;
}

CTile* Game::ptile (int x, int y)
{
if(mapa && tiles)
	{
	int	tx=x/tile_dx,
		ty=y/tile_dy;
	if(tx>=0 && ty>=0 && tx<mapa->width && ty<mapa->height)
		return mapa->tile(tx, ty);
	}
return 0;
}

bool Game::setMapTile (int x, int y, CTile ct)
{
if(mapa && tiles)
	{
	int	tx=x/tile_dx,
		ty=y/tile_dy;
	if(tx>=0 && ty>=0 && tx<mapa->width && ty<mapa->height)
		{
		*(mapa->tile(tx, ty)) = ct;
		setTile(tx, ty, ct);
		return true;
		}
	}
return false;
}

bool Game::mapCollides(int x, int y)
{
if(x<limit_x0 || y<limit_y0)
	return true;
if(x>=limit_x1 || y>=limit_y1)
	return true;
if(mapa)
	return mapa->collides(x/tile_dx, y/tile_dy);
return false;
}

int Game::mapCollides(const QRect &rec)
{
int	flags = 0;
int	cx = rec.left()+rec.width()/2;
int	cy = rec.top()+rec.height()/2;

if(mapCollides(cx, rec.top()))
	flags |= (CM_CENTER_UP);
if(mapCollides(cx, rec.bottom()))
	flags |= (CM_CENTER_DOWN);
if(mapCollides(rec.left(), cy))
	flags |= (CM_CENTER_LEFT);
if(mapCollides(rec.right(), cy))
	flags |= (CM_CENTER_RIGHT);
if(mapCollides(rec.left(), rec.top()))
	flags |= (CM_LEFT_UP);
if(mapCollides(rec.left(), rec.bottom()))
	flags |= (CM_LEFT_DOWN);
if(mapCollides(rec.right(), rec.top()))
	flags |= (CM_RIGHT_UP);
if(mapCollides(rec.right(), rec.bottom()))
	flags |= (CM_RIGHT_DOWN);
return flags;
}
