#include <stdio.h>
#include <stdlib.h>
#include "punq.h"
#include "bonus.h"
#include "mosca.h"
#include "oruga.h"
#include "carnivora.h"
#include "pulga.h"
#include "titulo.h"
#include "fuerza.h"
#include "estrella.h"
#include "scene.h"

#define	TPAUSA	100

ScePresenta	sce_presenta;
SceJuega	sce_juega;
SceExito	sce_exito;
SceFracaso	sce_fracaso;
SceFinal	sce_final;

#define	CURFILE	".punq"

// -----------------------------------------------------------

#define TILEDX	16
#define TILEDY	16

#define	MT_PIC	0x3F
#define	MT_HARD	0xFF
#define	MT_GATE	0x80

#define	B_NONE		'.'
#define	B_WALL		'O'
#define	B_GATE		'o'
#define	B_BONUS		'+'
#define	B_ORUGA		'x'
#define	B_PULGA		'p'
#define	B_MOSCA		'm'
#define	B_CARNIVORA	'c'

#define TILE_NX	15
#define TILE_NY	17

int	last_level=0;
int	cur_level=0;
Mapa	mapa;
MapaSet	theMapas;
QPixmap *qp_tiles =0;
bool	map_scroll;

// -----------------------------------------------------------

void clear_mapa()
{
mapa.alloc(TILE_NX, TILE_NY);
mapa.fill(0);
theGame->setMapa(&mapa);
theView->setView(2,2);
theGame->setLimits(theView->x0, theView->y0, theView->x1, theView->y1);
}

bool init_mapas()
{
qp_tiles = new QPixmap("tiles.png");
if(!qp_tiles)
	return false;
if(!theMapas.load("mapas.txt"))
	return false;
mapa.mask_test=MT_HARD;
mapa.mask_tile=MT_PIC;
mapa.mask_attr=0;
get_last_level();
return true;
}

// -----------------------------------------------------------

FILE* open_rc(const char* mode)
{
char fname[210];
#ifdef WIN32
strcpy(fname, CURFILE);
#else
char *userdir = getenv("HOME");
if(userdir && strlen(userdir)<200)
	sprintf(fname,"%s/%s", userdir, CURFILE);
else	strcpy(fname, CURFILE);
#endif
return fopen(fname, mode);
}

void get_last_level()
{
FILE *fp = open_rc("r");
last_level = 0;
if(fp)	{
	fscanf(fp, "%d", &last_level);
	fclose(fp);
	}
if(last_level<0 || last_level>theMapas.nmapas)
	last_level=0;
}

void set_last_level(int l)
{
FILE *fp = open_rc("w");
last_level = l;
if(fp)	{
	fprintf(fp, "%d\n", last_level);
	fclose(fp);
	}
}

// -----------------------------------------------------------

bool ScePresenta::enter()
{
set_info_level("");
theGame->setTilesGr(qp_tiles,TILEDY,TILEDY);
buttonsOff();
clear_mapa();
new Mosca(10,10);
new Pulga(100,10);
new Titulo("Punq\nthe Flea\n(La Pulga)");
eframe=0;
return true;
}

void ScePresenta::leave()
{
theGame->deleteAllItems();
if(last_level)
	cur_level=last_level;
else	cur_level=1;
}

void ScePresenta::advance()
{
if(eframe)
	{
	if(eframe == theGame->cur_frame)
		{
		if(thePulga->status == Pulga::REPOSO)
			{
			fuerza = (fuerza<0?1:-1)*(1+8*double(rand())/RAND_MAX);
			thePulga->ready(fuerza<0);
			eframe=theGame->cur_frame+20;
			}
		else if(thePulga->status == Pulga::PREPARADO)
			{
			thePulga->jump(fuerza);
			eframe = 0;
			}
		}
	}
else	{
	if(thePulga->status == Pulga::REPOSO)
		eframe=theGame->cur_frame+5+rand()%100;
	}
}

// -----------------------------------------------------------

bool SceJuega::enter()
{
char bf[32];
int i, j;
fgate = false;
sprintf(bf, "Level %d", cur_level);
set_info_level(bf);
buttonsOff();
mapa.copy(theMapas.mapas[cur_level-1]);
for(i=0; i<mapa.width; i++)
	for(j=0; j<mapa.height; j++)
		{
		CTile * pm =mapa.tile(i,j);
		switch(*pm)
			{
			case B_MOSCA:
				new Mosca(i*TILEDX, j*TILEDY);
				*pm = 0;
				break;
			case B_ORUGA:
				new Oruga(i*TILEDX, j*TILEDY);
				*pm = 0;
				break;
			case B_CARNIVORA:
				new Carnivora(i*TILEDX-7, j*TILEDY-15);
				*pm = 0;
				break;
			case B_PULGA:
				new Pulga(i*TILEDX, j*TILEDY);
				*pm = 0;
				break;
			case B_BONUS:
				new Bonus(i*TILEDX-3, j*TILEDY-4);
				*pm = 0;
				break;
			case B_NONE:
				*pm = 0;
				break;
			case B_GATE:
				fgate = true;
				*pm = (1+rand()%(theGame->ntiles-1))|MT_GATE;
				break;
			default:
				*pm = 1+rand()%(theGame->ntiles-1);
				break;
			}
		}
new Fuerza();
theGame->setMapa(&mapa);
if(mapa.width>TILE_NX || mapa.height>TILE_NY)
	{
	map_scroll=true;
	theView->setView(0,0);
	}
else	{
	map_scroll=false;
	theView->setView(2,0);
	theGame->setLimits(theView->x0, theView->y0, theView->x1, theView->y1);
	}
return true;
}

void SceJuega::leave()
{
bjump_pressed=false;
theGame->deleteAllItems();
}

#define	PMARGEN	8
#define	MARGEN	64
#define	SCROLL	4

void SceJuega::advance()
{
if(map_scroll)
	{
	bool f=false;
	int x=theView->x0, y=theView->y0;
	if(theView->mpress)
		{
		if(theView->mx<MARGEN)
			{ x=theView->x0-SCROLL; f=true; }
		if(theView->my<MARGEN)
			{ y=theView->y0-SCROLL; f=true; }
		if(theView->mx>(theView->dx-MARGEN))
			{ x=theView->x0+SCROLL; f=true; }
		if(theView->my>(theView->dy-MARGEN))
			{ y=theView->y0+SCROLL; f=true; }
		}
	else	{
		if(key_left)
			{ x=theView->x0-SCROLL; f=true; }
		if(key_up)
			{ y=theView->y0-SCROLL; f=true; }
		if(key_right)
			{ x=theView->x0+SCROLL; f=true; }
		if(key_down)
			{ y=theView->y0+SCROLL; f=true; }
		}
	if(f)	theView->setView(x, y, false);
	QRect rec = thePulga->boundingRect();
	rec.moveBy(-PMARGEN,-PMARGEN);
	rec.setWidth(rec.width()+PMARGEN*2);
	rec.setHeight(rec.height()+PMARGEN*2);
	theView->showRect(rec, false);
	theView->updateView();
	}
if(fgate && ! Bonus::count)
	{
	int i, j, pausa=0;
	fgate = false;
	for(i=0; i<mapa.width; i++)
		for(j=0; j<mapa.height; j++)
			{
			CTile *pm = mapa.tile(i,j);
			if(*pm & MT_GATE)
				{
//				theGame->setTile(i,j, *pm=0);
				new Estrella2(i*TILEDX, j*TILEDY, pausa);
				pausa += 20;
				}
			}
	}
}

// -----------------------------------------------------------

bool SceFracaso::enter()
{
buttonsOff();
clear_mapa();
new Titulo("Oooh!");
eframe=theGame->cur_frame+TPAUSA;
return true;
}

void SceFracaso::leave()
{
theGame->deleteAllItems();
}

void SceFracaso::advance()
{
if(eframe == theGame->cur_frame)
	theGame->setScene(&sce_juega);
}

// -----------------------------------------------------------

bool SceExito::enter()
{
buttonsOff();
clear_mapa();
new Titulo("Bravo!");
eframe=theGame->cur_frame+TPAUSA;
return true;
}

void SceExito::leave()
{
theGame->deleteAllItems();
}

void SceExito::advance()
{
if(eframe == theGame->cur_frame)
	{
	if(cur_level<theMapas.nmapas)
		{
		theGame->setScene(&sce_juega);
		set_last_level(++cur_level);
		}
	else	theGame->setScene(&sce_final);
	}
}

// -----------------------------------------------------------

bool SceFinal::enter()
{
buttonsOff();
clear_mapa();
new Titulo("THE\nEND");
eframe=theGame->cur_frame+TPAUSA;
set_last_level(0);
return true;
}

void SceFinal::leave()
{
theGame->deleteAllItems();
cur_level=0;
}

void SceFinal::advance()
{
if(eframe == theGame->cur_frame)
	theGame->setScene(&sce_presenta);
}

// -----------------------------------------------------------
